import { getBanList, setBanList } from "../../../roblox-open-cloud.js";
import { jsonError } from "../../../common.js";

export async function onRequestPost(context: RequestContext) {
  const { statsReduction } = context.data.body;

  if (statsReduction && typeof statsReduction !== "number")
    return jsonError("Invalid stat reduction", 400);

  const appeal = await context.env.DATA.get(
    `gameappeal_${context.params.id as string}`,
  );

  if (!appeal) return jsonError("Appeal not found", 400);

  const data = JSON.parse(appeal);
  const banList = (await getBanList(context)) as {
    [k: string]: { BanType: number; Unbanned?: boolean; UnbanReduct?: number };
  };

  await context.env.DATA.delete(`gameappeal_${context.params.id as string}`);
  await context.env.D1.prepare("DELETE FROM game_appeals WHERE id = ?;")
    .bind(context.params.id)
    .run();

  if (!banList[data.roblox_id])
    return new Response(null, {
      status: 204,
    });

  banList[data.roblox_id] = {
    BanType: 0,
    Unbanned: true,
    UnbanReduct: statsReduction,
  };

  await context.env.D1.prepare(
    "INSERT INTO game_mod_logs (action, evidence, executed_at, executor, id, target) VALUES (?, ?, ?, ?, ?, ?);",
  )
    .bind(
      "accept_appeal",
      `https://carcrushers.cc/mod-queue?id=${context.params.id}&type=gma`,
      Date.now(),
      context.data.current_user.id,
      crypto.randomUUID(),
      data.roblox_id,
    )
    .run();

  await setBanList(context, banList);

  return new Response(null, {
    status: 204,
  });
}
